package grm;

import grm.ExecutionManagerImpl.AppInfo;
import interfaces.Grm;
import interfaces.Lrm;
import interfaces.LrmHelper;

import java.util.Iterator;
import java.util.Map;

import org.omg.CORBA.ORB;

import subtypes.ApplicationType;
import subtypes.AsctRequestId;
import types.DistinctExecutionSpecs;

public class StandardFinishedExecutionManager {

    Map<String, AppInfo> currentExecutions;
    ORB orb;
    Grm grm;
    
    //--------------------------------------------------------------------------------------
    
    StandardFinishedExecutionManager(Map<String, AppInfo> currentExecutions, ORB orb, Grm grm) {
        this.currentExecutions = currentExecutions;
        this.orb = orb; this.grm = grm;
    }
    
    //--------------------------------------------------------------------------------------
    
    public int treatFinishedExecution(AppInfo appInfo, AsctRequestId asctRequestId, int restartId, int status) {

        if (status == 0)     processNormalExit(asctRequestId, appInfo);
        else if (status < 0) processAbnormalExit(asctRequestId, appInfo);
        else if (status > 0) processRestart(asctRequestId, appInfo);            

        return 0;
    }

    //--------------------------------------------------------------------------------------
    
    int processNormalExit(AsctRequestId asctRequestId, AppInfo appInfo) {

        appInfo.nodeAppIds.remove(Integer.parseInt(asctRequestId.appNodeRequestId));                
        
        /** Application is removed from currentExecutions */
        if (appInfo.nodeAppIds.size()==0) {
            System.out.println("ExecutionManagerImpl.reportExecutionFinished--> Execution Finished [" + asctRequestId.appMainRequestId + "]");
            currentExecutions.remove(asctRequestId.appMainRequestId);
        }
        
        return 0;
    }

    //--------------------------------------------------------------------------------------
    
    int processAbnormalExit(AsctRequestId asctRequestId, AppInfo appInfo) {

        /** Remove the reported node from the list, so that it will not be killed again */
        appInfo.nodeAppIds.remove(asctRequestId.appNodeRequestId);                   
        
        /** Kills all the remaining processes */
        Iterator<String[]> it = appInfo.nodeAppIds.values().iterator();                                     
        while (it.hasNext()) {
            String[] nodeLrmAppId = it.next();
            Lrm lrm = LrmHelper.narrow( orb.string_to_object( nodeLrmAppId[0] ));
            String appId = nodeLrmAppId[1];               
            if (ExecManDebug.Debug)
                System.err.println(">>>>> ExecutionManagerImpl.processAbnormalExit--> KILLING !!!!! " + " (" + appId + ")");
            lrm.kill(appId);
        }                               
        
        /** Application is removed from the ExecutionManager */
        System.out.println("ExecutionManagerImpl.reportExecutionFinished--> Execution finished [" + asctRequestId.appMainRequestId + "]");
        currentExecutions.remove(asctRequestId.appMainRequestId);                
        
        return 0;
    }

    //--------------------------------------------------------------------------------------
    
    int processRestart(AsctRequestId asctRequestId, AppInfo appInfo) {
        
      if (appInfo.commonSpecs.appType == ApplicationType.regular) {
          
        String[] nodeLrmAppId = appInfo.nodeAppIds.remove(Integer.parseInt(asctRequestId.appNodeRequestId));                            
        Lrm lrm = LrmHelper.narrow( orb.string_to_object( nodeLrmAppId[0] ));
        int ckpNumber = lrm.getLastCkpNumber(nodeLrmAppId[1]); 
        if (ckpNumber >=0) {
          String ckpName = " -r" + ckpNumber + " -d../" + nodeLrmAppId[1];
          appInfo.distinctSpecs[0].applicationArgs = ckpName;
        }
        grm.remoteExecutionRequest(appInfo.commonSpecs, appInfo.distinctSpecs, appInfo.forceDifferentMachines);
        
      }

      /** Removes the node from appInfo and reschedule the execution of the finished process */
      else if (appInfo.commonSpecs.appType == ApplicationType.parametric) {
          
          DistinctExecutionSpecs distinctSpecs = appInfo.distinctSpecs[Integer.parseInt(asctRequestId.appNodeRequestId)];
          String[] nodeLrmAppId = appInfo.nodeAppIds.remove(Integer.parseInt(asctRequestId.appNodeRequestId));                            
          Lrm lrm = LrmHelper.narrow( orb.string_to_object( nodeLrmAppId[0] ));
          int ckpNumber = lrm.getLastCkpNumber(nodeLrmAppId[1]); 
          if (ckpNumber >=0) {
              String ckpName = " -r" + ckpNumber + " -d../" + nodeLrmAppId[1];
              distinctSpecs.applicationArgs = ckpName;
          }                          
          grm.remoteExecutionRequest(appInfo.commonSpecs, new DistinctExecutionSpecs[] {distinctSpecs}, appInfo.forceDifferentMachines);
      }

        return 0;
    }

    //--------------------------------------------------------------------------------------
    
}
