#include "CkpStore.hpp"
#include "../../../shared/utils/c++/SocketUtils.hpp"

#include <iostream>
#include <sstream>
#include <cassert>

#include <strings.h>
#include <unistd.h>
#include <fcntl.h>

#define SERVER_HOST_ADDRESS "192.168.100.20"
#define SERVER_TCP_PORT 6002

CkpRepositoryStore::~CkpRepositoryStore() {
  delete serverAddress;
}

//---------------------------------------------------------------------------

CkpRepositoryStore::CkpRepositoryStore(std::string path, int delay) {

    std::string address;
    unsigned short port;
      
    std::istringstream repLocStream(path);
    repLocStream >> execNumber >> address >> port ;
    
    // Fills serverAddress with the server address and port
    serverAddress = new struct sockaddr_in;
    bzero((char *) serverAddress, sizeof(*serverAddress));
    serverAddress->sin_family      = AF_INET;
    serverAddress->sin_addr.s_addr = inet_addr(address.c_str());  
    serverAddress->sin_port        = htons(port);
    
    std::cout << "execNumber = " << execNumber << std::endl;
    std::cout << "address    = " << address    << std::endl;
    std::cout << "port       = " << port       << std::endl;

}

//---------------------------------------------------------------------------

int CkpRepositoryStore::connectToServer() {

  // Fills serverAddress with the server address and port
  //  struct sockaddr_in serverAddress;
  //  bzero((char *) &serverAddress, sizeof(serverAddress));
  //  serverAddress.sin_family      = AF_INET;
  //  serverAddress.sin_addr.s_addr = inet_addr(SERVER_HOST_ADDRESS);  
  //  serverAddress.sin_port        = htons(SERVER_TCP_PORT);
  
  // Opens a TCP socket
  int sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if ( sockfd < 0 )
    std::cerr << "CkpRepositoryStore::connectToServer -->" 
	      << " cannot open stream socket." << std::endl;

  // Connects to server
  // int connectFlag = connect (sockfd, (struct sockaddr *)&serverAddress, sizeof(serverAddress));
  int connectFlag = connect (sockfd, (struct sockaddr *)serverAddress, sizeof(*serverAddress));
  if ( connectFlag < 0 )
    std::cerr << "CkpRepositoryStore::connectToServer -->" 
	      << " cannot connect to server." << std::endl;
   
  return sockfd;
}

//---------------------------------------------------------------------------

void CkpRepositoryStore::saveData(void *data, long nbytes, int ckpNumber) {
  
  int sockfd = this->connectToServer();

  assert(ckpNumber >= 0);

  // sending a checkpoint 'CkpNumber' of 'nbytes'
  SocketUtils::writeUint32(sockfd, (long)1); // Indicates that a store will be performed
  //SocketUtils::writeToStream(sockfd, execNumber.c_str(), execNumber.length());
  SocketUtils::writeUint32(sockfd, (long)execNumber);
  SocketUtils::writeUint32(sockfd, (long)ckpNumber);
  SocketUtils::writeUint32(sockfd, (long)nbytes);

  // Transfer data to repository
  SocketUtils::writeToStream(sockfd, data, nbytes);

  std::ostringstream ckpFileName;
  ckpFileName << ckpNumber << ".ckp";
  long writeCompleted = SocketUtils::readUint32(sockfd);
  int outFile = creat(ckpFileName.str().c_str(),0600);
  close(outFile);
  
  std::cerr << "Finished sending data!!! [" << writeCompleted << "]" << std::endl;

  close (sockfd);

}

//---------------------------------------------------------------------------

void CkpRepositoryStore::recoverCkpData(void * & data, long & dataSize, int ckpNumber) {
  
  int sockfd = this->connectToServer();
  
  // Asking for checkpoint 'CkpNumber' 
  SocketUtils::writeUint32(sockfd, (long)2);
  SocketUtils::writeUint32(sockfd, (long)execNumber);
  SocketUtils::writeUint32(sockfd, (long)ckpNumber);

  // Reads dataSize from the stream
  dataSize = SocketUtils::readUint32(sockfd);
    
  // Reads the file data from the stream to 'data'
  data = malloc(dataSize);
  SocketUtils::readFromStream(sockfd, data, dataSize);

  close (sockfd);
  std::cerr << "Finished recovering data!!!" << std::endl;

}

//---------------------------------------------------------------------------

