#include <cassert>

extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}

#include "utils/c++/LuaUtils.hpp"
#include "utils/c++/OrbUtils.hpp"
#include "utils/c++/StringUtils.hpp"

#include "ExecutionManagerStub.hpp"

#include <iostream>
using std::cerr;
using std::endl; 


  //----------------------------------------------------------------------
  ExecutionManagerStub::ExecutionManagerStub(lua_State * aState,
                     const string & execManagerIor):state(aState){
    //FIXME: This assumes that state ALREADY loaded the IDL for the GRM. Due to
    //O2's current design it is not possible to load the GRM IDL separately.
    OrbUtils::instantiateProxy (state, execManagerIor, "IDL:interfaces/ExecutionManager:1.0",
                               "execManagerProxy");
  }

  //--------------------------------------------------------------------------------
  void ExecutionManagerStub::reportExecutionStarted
  (const string & appMainRequestId, const string & appNodeRequestId, const string & lrmIor, 
   const int & restartId, const string & appId) {

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "execManagerProxy");
    lua_pushstring(state, "reportExecutionStarted");
    lua_gettable(state, -2);
    lua_getglobal(state, "execManagerProxy");
    lua_pushstring(state, lrmIor.c_str());       
    lua_pushstring(state, appId.c_str()); 
    lua_pushnumber(state, restartId);
    lua_newtable(state);
    LuaUtils::setFieldOnTable(state, "appMainRequestId", appMainRequestId.c_str(), -1);
    LuaUtils::setFieldOnTable(state, "appNodeRequestId", appNodeRequestId.c_str(), -1);
    if (lua_pcall(state, 5, 0, 0) != 0){
      cerr << "[ERROR] ExecutionManagerStub::reportExecutionStarted->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }

    lua_pop(state, 1);//removing execManagerProxy
    assert(stackTop == lua_gettop(state));
  }

  //--------------------------------------------------------------------------------
  int ExecutionManagerStub::reportExecutionFinished
  (const string & appMainRequestId, const string & appNodeRequestId, 
   const int & restartId, int exitStatus) {

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "execManagerProxy");
    lua_pushstring(state, "reportExecutionFinished");
    lua_gettable(state, -2);
    lua_getglobal(state, "execManagerProxy");
    lua_pushnumber(state, restartId);
    lua_newtable(state);
    LuaUtils::setFieldOnTable(state, "appMainRequestId", appMainRequestId.c_str(), -1);
    LuaUtils::setFieldOnTable(state, "appNodeRequestId", appNodeRequestId.c_str(), -1);
    lua_pushnumber(state, exitStatus);
    if (lua_pcall(state, 4, 1, 0) != 0){
      cerr << "[ERROR] ExecutionManagerStub::reportExecutionFinished->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }
  
    int flag = (int)lua_tonumber(state, -1);
    lua_pop(state, 2);//removing asctProxy + result
    assert(stackTop == lua_gettop(state));

    return flag;
  }
