#include "LocDynamicInfo.hpp"

#include <sys/sysinfo.h>
#include <sys/vfs.h>
#include <iostream>
#include <sstream>
#include <cmath>

using namespace std;

LocDynamicInfo::LocDynamicInfo(LocStaticInfo & _lsi,float _threshold):
                                                                threshold(_threshold),
                                                                lsi(_lsi),
                                                                cpuUsage(CpuUsage::getInstance()){

 update();
 update();//Initializes prev and current
}

void LocDynamicInfo::update(){

  //cpuUsage = CpuUsage::getInstance();
  //Initializes with information obtained from sysinfo(man 2 sysinfo)
  //FIXME: Only work in 2.3+ kernels


  struct sysinfo sysInfo;
    if(sysinfo(&sysInfo)){
     cerr << "LocDynamicInfo::Error on call to sysinfo" << endl;
     exit(1);
  }

  prevFreeRAM = freeRAM;
  freeRAM =  ((sysInfo.freeram + sysInfo.bufferram) * sysInfo.mem_unit) / 1024;
  prevFreeSwap = freeSwap;
  freeSwap = (sysInfo.freeswap * sysInfo.mem_unit) / 1024;

  //Get disk information using statfs
  //FIXME: COMM from 2k:
  //FIXME: Don't Think about large file system (more than 32bit)

  struct statfs vfs;

  if(statfs(".", &vfs)){
    cerr << "LocDynamicInfo::Error on call to statfs" << endl;
    exit(1);
  }

  prevFsFree = fsFree;
  fsFree = (vfs.f_bavail / 1024) * (vfs.f_bsize / 1024) + 1;
  prevFilesFree = filesFree;
  filesFree = vfs.f_ffree;

  prevFsTotal = fsTotal;
  fsTotal = ((vfs.f_blocks - (vfs.f_bfree - vfs.f_bavail)) / 1024) * (vfs.f_bsize / 1024) + 1;

  prevFilesTotal = filesTotal;
  filesTotal = vfs.f_files;
  prevCpuUsage = currCpuUsage;
  currCpuUsage = cpuUsage.utilization();
}


 void LocDynamicInfo::testChange(){


   cout << "LocDynamicInfo::testChange " << endl << endl;
   cout << "freeram:      " << freeRAM << endl;
   cout << "prevFreeRAM:      " << prevFreeRAM << endl;
   cout << "freeswap:     " << freeSwap << endl;
   cout << "prevFreeSwap:     " << prevFreeSwap << endl;
   cout << "currCpuUsage:     " <<  currCpuUsage << endl;
   cout << "prevCpuUsage:     " <<prevCpuUsage  << endl;

   if((fabs(((double) freeRAM - prevFreeRAM) ) * 100 ) / lsi.getTotalRAM()  > threshold)
      cout << "RAM changed" << endl ;
   if((fabs(((double) freeSwap - prevFreeSwap) ) * 100 ) / lsi.getTotalSwap() > threshold)
      cout << "SWAP changed" << endl ;
   //FIXME: Old code was
   //if((fabs(((double) currCpuUsage - prevCpuUsage )) * 100 )  > threshold)
   //But this causes a LOT of updates, i.e. if cpu varies from 5 % to 10 %,
   //it will update even for a threshold of 50

   if((fabs(((double) currCpuUsage - prevCpuUsage )) ) > threshold)
      cout << "CPU USAGE changed" << endl ;

 }


  bool LocDynamicInfo::hadSignificantChange(){
    update();
    //testChange();//FIXME: REMOVE. For debugging only

    if((fabs(((double) freeRAM - prevFreeRAM) ) * 100 ) / lsi.getTotalRAM()  > threshold)
      return true;
    else if((fabs(((double) freeSwap - prevFreeSwap )) * 100 ) / lsi.getTotalSwap() > threshold)
      return true;
    //FIXME: Old code was
    //if((fabs(((double) currCpuUsage - prevCpuUsage )) * 100 )  > threshold)
    //But this causes a LOT of updates, i.e. if cpu varies from 5 % to 10 %,
    //it will update even for a threshold of 50
    else if((fabs(((double) currCpuUsage - prevCpuUsage ))) > threshold)
      return true;
    return false;
  }

  string LocDynamicInfo::strUtilization() const{
    ostringstream temp;
    temp << currCpuUsage;
    string cpu = temp.str();
    return cpu;
  }



