package asctGui;

import interfaces.ApplicationRepository;
import interfaces.ApplicationRepositoryPackage.ApplicationNotFoundException;
import interfaces.ApplicationRepositoryPackage.ApplicationRegistrationException;
import interfaces.ApplicationRepositoryPackage.BinaryNotFoundException;
import interfaces.ApplicationRepositoryPackage.DirectoryCreationException;
import interfaces.ApplicationRepositoryPackage.DirectoryNotEmptyException;
import interfaces.ApplicationRepositoryPackage.DirectoryNotFoundException;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.Stack;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import types.AppDescription;
import types.BinaryDescription;
import types.ContentDescription;

/*
 * Created on Sep 16, 2004
 */

/**
 * @author vidal
 */
public class RepositoryPanel extends JPanel implements ActionListener{

	private DefaultMutableTreeNode rootNode;
	private DefaultMutableTreeNode lastSelectedNode;

	private final RepositoryTree treeView;	// provides a view of the data
	private DefaultTreeModel treeModel;	//

	private Asct asct;
	private JFrame frame;

	// ***********************************************************************************
	//	initializing methods
	// ***********************************************************************************
	/**
	 * constructor
	 * @param String path
	 * @param Asct asct
	 */
	public RepositoryPanel(String pathName, Asct asct) {
		this.asct = asct;
		Icon[] icons = new Icon[6];
		icons[0]  = createImageIcon("../images/root.gif");
		icons[1]  = createImageIcon("../images/appDir.gif");
		icons[2]  = createImageIcon("../images/appDescFile.gif");
		icons[3]  = createImageIcon("../images/commonDir.gif");
		icons[4]  = createImageIcon("../images/commonFile.gif");
		icons[5]  = createImageIcon("../images/binaryFile.gif");

		
		rootNode = createNode( "/","_root",true );	// creates the root node
		
		lastSelectedNode = rootNode;

		treeView = new RepositoryTree(rootNode,this);		// creates a JT0ree
		treeView.setCellRenderer(new RepositoryRenderer(icons));

		buildRepositoryTree(rootNode); // creates all nodes recursively

		
		treeModel = new DefaultTreeModel(rootNode, true);
		treeModel.addTreeModelListener(new RepositoryTreeModelListener());
		

		treeView.setModel(treeModel);
		treeView.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		treeView.setRootVisible(true);
		treeView.setEditable(false);
		treeView.addMouseListener (new RepositoryTreeMouseAdapter(treeView));
		treeView.addTreeSelectionListener(new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)
						   treeView.getLastSelectedPathComponent();
					if(node == null)
						return;
					lastSelectedNode = node;
				}
		});

		//Enable tool tips.
		ToolTipManager.sharedInstance().registerComponent(treeView);
		
		JScrollPane treeScroll = new JScrollPane(treeView);
		treeScroll.setPreferredSize(new Dimension(200,200));

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setLeftComponent(treeScroll);

		this.setLayout(new BorderLayout());
		this.add(splitPane, BorderLayout.CENTER);
	}//  RepositoryPanel constructor


	/**
	 * @author vidal
	 * @param DefaultMutableTreeNode rootNode
	 * creates the nodes present in the tree
	 * a method adapted from 'FileViewPanel.createTree' method in FileViewPanel.java file
	 */
	private void buildRepositoryTree( DefaultMutableTreeNode parentNode) {
		DefaultMutableTreeNode idxNode;
		boolean allowsChildren;
		TreeNamePathHolder nodeDesc = (TreeNamePathHolder) parentNode.getUserObject();
		
		if (nodeDesc.getKind().lastIndexOf("Dir") > -1 || nodeDesc.getKind().equals("_root") ) {
			ContentDescription contents[]=null;
			try{
				contents = asct.getApplicationRepositoryWrapper().listContents(nodeDesc.getFilePath());
				
			}catch(Exception e)
			{
				System.out.println(e.toString());
			}
			
			for(int i = 0; i < contents.length; i++) {
				allowsChildren = contents[i].kind.lastIndexOf("Dir") > -1;
				idxNode = createNode(contents[i].filename,  contents[i].kind, allowsChildren);
				parentNode.add(idxNode);
				buildRepositoryTree(idxNode);
			}
				
			}
	}// rebuildRepositoryTree method
	

	/** Returns an ImageIcon, or null if the path was invalid. */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = RepositoryRenderer.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}// createImageIcon method


	// ***********************************************************************************
	//	auxiliary methods
	// ***********************************************************************************

	public JFrame getFrame() {
		return frame;
	}

	/**
	 * @author vidal
	 * @param String nodeName
	 * @param String kind - kind of this node
	 * @param boolean allowsChildren
	 */
	
	
	 public  DefaultMutableTreeNode createNode(String nodeName , String kind, boolean allowsChildren) {
			String platform = "";
			String fileName = nodeName;
			String filePath = nodeName;
		 	if (kind.equals("_binaryFile") ){
				int kindNodeIndex = fileName.lastIndexOf("_");
		 		platform = fileName.substring(0, kindNodeIndex );
		 		fileName = platform;
		 	}
			 DefaultMutableTreeNode node = new DefaultMutableTreeNode( 
			 	new TreeNamePathHolder(filePath, fileName, kind, platform) ); 
			 node.setAllowsChildren(allowsChildren);
			 return node;
		 }// createNode method

	/**
 	* @author vidal
 	* @param DefaultMutableTreeNode childNode
 	* @param DefaultMutableTreeNode parentNode
 	*/
 	public void insertNode(DefaultMutableTreeNode childNode, DefaultMutableTreeNode parentNode) {
		treeModel.insertNodeInto(childNode, parentNode, parentNode.getChildCount());
		treeModel.nodeChanged(childNode);
	}

	/**
	 * @author vidal
	*/
	public void removeNode() {
		treeModel.removeNodeFromParent(lastSelectedNode);
	}




	// ***********************************************************************************
	//	action executing methods
	// ***********************************************************************************
	/**
	 * @author vidal
	 * @param String parent
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void createDirectory(String parent) {
		String dirName = "";
		dirName = (String) JOptionPane.showInputDialog(frame,"Enter the name of the new Directory",
			"Create Directory",JOptionPane.PLAIN_MESSAGE,null,null,"");
		if(dirName != null && dirName.length() != 0) {
			String kind = "_commonDir";
			String dir = parent + "/" + dirName + kind;
			try {
				//updates the application repository
				
				asct.getApplicationRepositoryWrapper().createDirectory(dir);
				
				// creates a common directory node
				boolean allowsChildren = true;
				DefaultMutableTreeNode commonDirNode = createNode(dir, kind, allowsChildren);
				// adds this common directory node at the tree
				DefaultMutableTreeNode parentNode = null;
				TreePath parentPath = treeView.getSelectionPath();
				if (parentPath == null) {
					parentNode = rootNode; //	if there's no selection. Default to the root node.
				}
				else {
					parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());
				}
				insertNode( commonDirNode, parentNode );
			} catch (DirectoryCreationException e) {
					JOptionPane.showMessageDialog(frame, e.myMessage, 
						"Exception Warning",JOptionPane.WARNING_MESSAGE);
			}
		}
	}//createDirectory method


	/**
	 * @author vidal
	 * @param String parentName
	 * @param String parentKind
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void registerApplication(String parentName, String parentKind) {
		String appName = "";
		appName = (String) JOptionPane.showInputDialog(
			frame, 
			"Enter the name of the new Application",
			"Register an Application",
			JOptionPane.PLAIN_MESSAGE,
			null,null,"");
		if(appName != null && appName .length() != 0) {
			String appDirName = parentName + "/" + appName + "_appDir";
			String appDescName = "/AppDescription_appDescFile";
			try {
				//updates the application repository
				asct.getApplicationRepositoryWrapper().registerApplication(appDirName, appDescName);
				//creates an appDir node  
				boolean allowsChildren = true;
				DefaultMutableTreeNode dirNode = createNode(appDirName, "_appDir", allowsChildren);
				// adds this appDir node at the tree
				DefaultMutableTreeNode parentNode = null;
				TreePath parentPath = treeView.getSelectionPath();
				if (parentPath == null) {
					// if there's no selection. Default to the root node.
					parentNode = rootNode;
				}
				else {
					parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());
				}
				insertNode(dirNode, parentNode);
				//creates an appDesc node
				allowsChildren = false;
				DefaultMutableTreeNode appDescNode = createNode(appDescName, "_appDescFile", allowsChildren);
				// adds this appDesc node at the tree
				insertNode(appDescNode, dirNode);
			} catch (ApplicationRegistrationException e) {
				JOptionPane.showMessageDialog(frame, e.myMessage, 
					"Exception Warning",JOptionPane.WARNING_MESSAGE);
			} catch (DirectoryCreationException e) {
				JOptionPane.showMessageDialog(frame, e.getMessage(), 
					"Exception Warning",JOptionPane.WARNING_MESSAGE);
			}
		}//endif
	}// registerApplication method


	/**
	 * @author vidal
	 * @param String appDirName
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void uploadBinary(String appDirName) {
		asct.uploadBinary(this, appDirName);
	}
	
	/**
	 * @author vidal
	 * @param BinaryDescription binDesc
	 * called from UploadPanel. getBinary method
	 */
	void insertBinaryNode(BinaryDescription binDesc) {
		String binaryNodeName =  binDesc.dirname + "/" + binDesc.binname;
		boolean allowsChildren = false;
		DefaultMutableTreeNode binaryNode = createNode(binaryNodeName, "_binaryFile", allowsChildren);
		TreeNamePathHolder descNode =   (TreeNamePathHolder )binaryNode.getUserObject();
		descNode.setPlatform(binDesc.platform);
		// adds this binary node at the tree
		TreePath parentPath = treeView.getSelectionPath();
		DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());
		insertNode(binaryNode, parentNode);				
	}// insertBinaryNode method


	/**
	 * @author vidal
	 * @param String parentName
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void unregisterApplication(String parentName) {
		String appName  = "/AppDescription_appDescFile";
		try { 
			int returnVal = JOptionPane.showConfirmDialog(frame,"Do you really want unregister this Application ?",
					"Unregister this Application", JOptionPane.OK_CANCEL_OPTION );
			if(returnVal == JOptionPane.OK_OPTION) {
				//updates the application repository
				asct.getApplicationRepositoryWrapper().unregisterApplication(parentName,appName);
				
				//	updates the tree view
				removeNode();
			}
		} catch (DirectoryNotEmptyException e) {
			JOptionPane.showMessageDialog(frame, 
				e.getMessage()+" " +e.myMessage, 
				"Exception Warning",JOptionPane.WARNING_MESSAGE);
		} catch (DirectoryNotFoundException e) {
			JOptionPane.showMessageDialog(frame, 
				e.getMessage()+" " +e.myMessage, 
				"Exception Warning",JOptionPane.WARNING_MESSAGE);
		} catch (ApplicationNotFoundException e) {
			JOptionPane.showMessageDialog(frame, 
				e.getMessage()+" " +e.myMessage, 
				"Exception Warning",JOptionPane.WARNING_MESSAGE);
		}
	}// unregisterApplication method


	/**
	 * @author vidal
	 * @param removableDir
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void removeDirectory(String removableDir) {
		try { 
			//TODO not to permit remove the root directory
			int returnVal = JOptionPane.showConfirmDialog(frame,"Do you really want remove this Directory ?",
				"Remove Directory", JOptionPane.OK_CANCEL_OPTION);
			if(returnVal == JOptionPane.OK_OPTION) {
				//	removes this directory from the application repository
				asct.getApplicationRepositoryWrapper().removeDirectory(removableDir);
				//	updates the tree view
				removeNode() ;
			}
		}catch (DirectoryNotEmptyException e) {
			JOptionPane.showMessageDialog(frame, e.getMessage(),
					"Exception Warning",JOptionPane.WARNING_MESSAGE);
		} catch (DirectoryNotFoundException e) {
			JOptionPane.showMessageDialog(frame, e.getMessage(),
					"Exception Warning",JOptionPane.WARNING_MESSAGE);
		}
	}// removeDirectory method
	
	
	/**
	 * @author vidal
	 * @param String absolutePath
	 * @param String platform
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void deleteBinary(String absolutePath, String platform) {
		String dirName = absolutePath.substring(0, absolutePath.lastIndexOf("/")+1);
		String binaryName = absolutePath.substring(absolutePath.lastIndexOf("/")+1);
		
		int returnVal = JOptionPane.showConfirmDialog(frame,"Do you really want delete this Binary file ?",
			"Delete Binary", JOptionPane.OK_CANCEL_OPTION);

		if(returnVal == JOptionPane.OK_OPTION) {
			try {
				//deletes this binary file from the repository
				asct.getApplicationRepositoryWrapper().deleteApplicationBinary(dirName, binaryName, platform);
				//	updates the tree view
				removeNode();
			} catch (ApplicationNotFoundException e) {
				JOptionPane.showMessageDialog(frame, e.getMessage()
				+e.myMessage, "Exception Warning",
				JOptionPane.WARNING_MESSAGE);
			} catch (DirectoryNotFoundException e) {
				JOptionPane.showMessageDialog(frame, e.getMessage()
				+e.myMessage, "Exception Warning",
				JOptionPane.WARNING_MESSAGE);
			} catch (BinaryNotFoundException e) {
				JOptionPane.showMessageDialog(frame, e.getMessage()
				+e.myMessage, "Exception Warning",
				JOptionPane.WARNING_MESSAGE);
			}
		}
	}// deleteBinary method


	/**
	 * @author vidal
	 * @param String absolutePath
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void executeApplication(String absolutePath, String platform) {
		String appId = absolutePath;
		String appPath = absolutePath;
		boolean allBinary = true;
		asct.requestExecution(appPath, appId, allBinary); 
	}

	/**
	 * @author vidal
	 * @param String absolutePath
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void executeBinary(String absolutePath, String platform) {
		String appPath = absolutePath.substring(0, absolutePath.lastIndexOf("/")+1);
		String appId = absolutePath.substring(absolutePath.lastIndexOf("/")+1);
		boolean allBinary = false;
		asct.requestExecution(appPath, appId, allBinary);
	}

	/**
	 * @author vidal
	 * @param appDescName
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void viewFileDescription(String absolutePath) {
		AppDescription appDesc=null;
		String appPath = absolutePath.substring(0,absolutePath.lastIndexOf("/")+1);
		String appDescFile = absolutePath.substring(absolutePath.lastIndexOf("/")+1);
		try {
			appDesc = asct.getApplicationRepositoryWrapper().getAppDescription(appPath);
		} catch (ApplicationNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DirectoryNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		asct.viewDescription(appDesc);
	}

	/**
	 * @author vidal
	 * @param appDescName
	 * called from RepositoryTreeMouseAdapter. processMouseEvent method
	 */
	void editFile(String appDescName) {
		JOptionPane.showMessageDialog(frame, "under construction. Be patient !");
	}


	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		JComboBox cb = (JComboBox)e.getSource();
		String osName = (String)cb.getSelectedItem();
	}
}// class RepositoryPanel




/*============================================================
/*auxiliary classes 
/*============================================================
*/
class RepositoryTreeMouseAdapter extends MouseAdapter{
	private RepositoryTree treeView;

	/**
	 * @author Andrei
	 * constructor
	 */
	RepositoryTreeMouseAdapter(RepositoryTree aTreeView) { treeView = aTreeView; }

	/**
	 * @author vidal
	 * @param abspath
	 */
	private String getmyPath(Object[] abspath) {
		int i = 0;
		String path = abspath[i].toString();
		if(abspath.length == 1)
			return path;
		for(i = 1; i < abspath.length-1;i++) {
			path = path + "/" +abspath[i].toString();			
		}
		return path + "/" + abspath[i].toString(); 
	}// getmyPath method
	
	/**
	 * @author vidal
	 * @param abspath
	 */
	private String getmyParent(Object[] abspath) {
		int i = 0;
		String path = abspath[i].toString();
		for(i = 1; i < abspath.length-1;i++) {
			path = path + "/" +abspath[i].toString();			
		}
		return path + "/"; 
	}// getmyParent method
	
	/**
	 * @author vidal
	 * @param MouseEvent e
	 */
	private void processMouseEvent(MouseEvent e) {
		if(e.getComponent().getClass() == RepositoryTree.class) {
			TreePath selPath = treeView.getPathForLocation(e.getX(), e.getY());
			if(selPath != null) {
				if(selPath.getLastPathComponent().getClass() == DefaultMutableTreeNode.class) {
					treeView.setSelectionPath(selPath);	// it sets the clicked node as selected
					final TreeNamePathHolder selectedPathHolder = ((TreeNamePathHolder)((DefaultMutableTreeNode)selPath.getLastPathComponent()).getUserObject());

					if(selectedPathHolder.getKind().equals("_root")) {
						final String absolutePath = selectedPathHolder.getFilePath();
						
						JMenuItem registerAppItem = new JMenuItem("Register Application");
						registerAppItem.setToolTipText("Register a new application under this selected directory");
						registerAppItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().registerApplication(absolutePath, "_root");
									}
						});
						JMenuItem createDirItem = new JMenuItem("Create Directory");
												createDirItem.setToolTipText("Create a new directory under this selected directory");
												createDirItem.addActionListener(new ActionListener() {
															public void actionPerformed(ActionEvent e) {
																treeView.getMyPanel().createDirectory(absolutePath);
															} 
												});
							
						JPopupMenu popup = new JPopupMenu ();
						popup.add (registerAppItem);
						popup.add (createDirItem);
						popup.setInvoker (e.getComponent());
						popup.show (e.getComponent(), e.getX(), e.getY());
					}
					else if(selectedPathHolder .getKind().equals("_appDir")) {
						final String absolutePath = selectedPathHolder.getFilePath();
						//selectedPathHolder .
						JMenuItem executeAppItem = new JMenuItem("Execute Application");
						//executeAppItem.s
						executeAppItem.setToolTipText("Execute  any possible application binary");
						executeAppItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().executeApplication(absolutePath, selectedPathHolder.getPlatform());
									} 
						});
						JMenuItem uploadBinaryItem = new JMenuItem("Upload Binary");
						uploadBinaryItem.setToolTipText("Upload a binary code for this application");
						uploadBinaryItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().uploadBinary(absolutePath);
									}
						});
						JMenuItem registerAppItem = new JMenuItem("Register Application");
						registerAppItem.setToolTipText("Register a new application at the Application Repository");
						registerAppItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().registerApplication(absolutePath, "_appDir");
									}
						});
						JMenuItem createDirItem = new JMenuItem("Create Directory");
						createDirItem.setToolTipText("Create a new directory under this selected directory");
						createDirItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().createDirectory(absolutePath);
									} 
						});
						JMenuItem unregisterAppItem = new JMenuItem("Unregister Application");
						unregisterAppItem.setToolTipText("Unregister an application at the Application Repository");
						unregisterAppItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().unregisterApplication(absolutePath);
									} 
						});
						JPopupMenu popup = new JPopupMenu ();
						popup.add (uploadBinaryItem);
						popup.add (executeAppItem);
						popup.add (registerAppItem);
						popup.add (createDirItem);
						popup.add (unregisterAppItem);
						popup.setInvoker (e.getComponent());
						popup.show (e.getComponent(), e.getX(), e.getY());
					}
					else if(selectedPathHolder .getKind().equals("_commonDir")) {
						final String absolutePath = selectedPathHolder.getFilePath();
						
						JMenuItem registerAppItem = new JMenuItem("Register Application");
						registerAppItem.setToolTipText("Register a new application at the Application Repository");
						registerAppItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().registerApplication(absolutePath,"_commonDir");
									}
						});
						JMenuItem createDirItem = new JMenuItem("Create Directory");
						createDirItem.setToolTipText("Create a directory under the current directory");
						createDirItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().createDirectory(absolutePath);
									} 
						});							
						JMenuItem removeDirItem = new JMenuItem("Remove this Directory");
						removeDirItem.setToolTipText("Remove this selected directory");
						removeDirItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().removeDirectory(absolutePath);
									} 
						});						
						JPopupMenu popup = new JPopupMenu ();
						popup.add (registerAppItem);
						popup.add (createDirItem);
						popup.add (removeDirItem);
						popup.setInvoker (e.getComponent());
						popup.show (e.getComponent(), e.getX(), e.getY());
					}
					else if(selectedPathHolder .getKind().equals("_appDescFile")) {
						final String absolutePath = selectedPathHolder .getFilePath();
							
						JMenuItem viewFileItem = new JMenuItem("View this File");
						viewFileItem.setToolTipText("View this selected application Description File");
						viewFileItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().viewFileDescription(absolutePath);
									} 
						});
						JPopupMenu popup = new JPopupMenu ();
						popup.add (viewFileItem);
						popup.setInvoker (e.getComponent());
						popup.show (e.getComponent(), e.getX(), e.getY());
					}
					else if(selectedPathHolder .getKind().equals("_binaryFile")) {
						final String absolutePath = selectedPathHolder .getFilePath();

						JMenuItem executeBinaryItem = new JMenuItem("Execute");
						executeBinaryItem.setToolTipText("Execute this selected application binary");
						executeBinaryItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().executeBinary(absolutePath, selectedPathHolder.getPlatform() );
									} 
						});
						JMenuItem deleteBinaryItem = new JMenuItem("Delete");
						deleteBinaryItem.setToolTipText("Delete this selected application binary");
						deleteBinaryItem.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										treeView.getMyPanel().deleteBinary(absolutePath, selectedPathHolder.getPlatform() );
									} 
						});					
						JPopupMenu popup = new JPopupMenu ();
						popup.add (executeBinaryItem);
						popup.add (deleteBinaryItem);
						popup.setInvoker (e.getComponent());
						popup.show (e.getComponent(), e.getX(), e.getY());
					}
				}
			}
		}
	}//processMouseEvent method

	public void mousePressed (MouseEvent e) { processMouseEvent(e); }

	public void mouseClicked (MouseEvent e) { processMouseEvent(e);}
	
	public void mouseReleased (MouseEvent e) { processMouseEvent(e); }
	
}// class RepositoryTreeMouseAdapter


//=========================================================================
class RepositoryRenderer extends DefaultTreeCellRenderer {
	private Icon rootDirIcon;
	private Icon appDirIcon;
	private Icon appDescIcon;
	private Icon commonDirIcon;
	private Icon commonFileIcon;
	private Icon binaryIcon;
	
	public RepositoryRenderer(Icon [] images) {
		super();
		rootDirIcon = images[0];
		appDirIcon = images[1];
		appDescIcon = images[2];
		commonDirIcon = images[3];
		commonFileIcon = images[4];
		binaryIcon = images[5];		
	}
	
	public Component getTreeCellRendererComponent(
	   	JTree tree,
	   	Object value,
	   	boolean sel,
	   	boolean expanded,
	   	boolean leaf,
	   	int row,
	   	boolean hasFocus) {
		
	   	super.getTreeCellRendererComponent(
			   tree, value, sel,
			   expanded, leaf, row,
			   hasFocus);
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
		if (isBinaryNode(node)) {
			setIcon(binaryIcon);
		}else if (isRootNode(node)) {
			setIcon(rootDirIcon);
		}else if (isAppDirNode(node)) {
			setIcon(appDirIcon);
		}else if (isAppDescNode(node)) {
			setIcon(appDescIcon);
		}else if (isCommonDirNode(node)) {
			//setIcon(appDescIcon);
		}else if (isCommonFileNode(node)) {
			//setIcon(appDescIcon);
		}
	   return this;
	}// getTreeCellRendererComponent method

	protected boolean isCommonDirNode(DefaultMutableTreeNode node) {
		if( node.getUserObject().getClass() == TreeNamePathHolder.class) {
			TreeNamePathHolder pathHolder = (TreeNamePathHolder)( node.getUserObject() );
			return (pathHolder.isCommonDir());
		}
		return false;
	}// isCommonDirNode method

	protected boolean isCommonFileNode(DefaultMutableTreeNode node) {
		if( node.getUserObject().getClass() == TreeNamePathHolder.class) {
			TreeNamePathHolder pathHolder = (TreeNamePathHolder)( node.getUserObject() );
			return (pathHolder.isCommonFile());
		}
		return false;
	}// isCommonFileNode method


	protected boolean isBinaryNode(DefaultMutableTreeNode node) {
		if( node.getUserObject().getClass() == TreeNamePathHolder.class) {
			TreeNamePathHolder pathHolder = (TreeNamePathHolder)( node.getUserObject() );
			return (pathHolder.isBinary());
		}
		return false;
	}// isBinaryNode method
	
	protected boolean isRootNode(DefaultMutableTreeNode node) {
		if( node.getUserObject().getClass() == TreeNamePathHolder.class) {
			TreeNamePathHolder pathHolder = (TreeNamePathHolder)( node.getUserObject() );
			return (pathHolder.isRoot());
		}
		return false;
	}// isRootNode method

	protected boolean isAppDescNode(DefaultMutableTreeNode node) {
		if( node.getUserObject().getClass() == TreeNamePathHolder.class) {
			TreeNamePathHolder pathHolder = (TreeNamePathHolder)( node.getUserObject() );
			return (pathHolder.isAppDesc());
		}
		return false;		
	}// isAppDescNode method

	protected boolean isAppDirNode(DefaultMutableTreeNode node) {
		if( node.getUserObject().getClass() == TreeNamePathHolder.class) {
			TreeNamePathHolder pathHolder = (TreeNamePathHolder)( node.getUserObject() );
			return (pathHolder.isApp());
		}
		return false;
	}// isAppDirNode method
	
}// class MyRenderer


//=========================================================================
class RepositoryTree extends JTree {

	private RepositoryPanel myPanel;

	/**
	 * @author vidal
	 */
	public RepositoryTree(DefaultMutableTreeNode rootNode, RepositoryPanel myPanel) {
		super(rootNode,true);
		this.myPanel = myPanel;
	}

	public RepositoryPanel getMyPanel(){return myPanel;}

}//class RepositoryTree


//=========================================================================
class TreeNamePathHolder {
	private String fileName;
	private String filePath;
	private String nodeKind;
	private String nodeName;
	private String nodePath;
	private Vector pathVector;
	private String binaryPlatform;
	
	TreeNamePathHolder(String filePath, String fileName, String kind, String binaryPlatform) {
		this.filePath = filePath;
		this.fileName = fileName;
		nodeKind = kind;
		pathVector = new Vector();
		nodePath = fileToNode(filePath.substring(filePath.indexOf("/")));
		
		String lastName = (String) pathVector.firstElement();
		if(!filePath.equals("/"))
		{
			nodeName = lastName.substring(0,lastName.lastIndexOf("_"));
		}else{
			nodeName = "/";
		}
		this.binaryPlatform = binaryPlatform;
	}

	/**
	 * @author vidal
	 * @param String path - substring de fileNode.getPath()
	 * creates an string - nodePath,  with the name path of the node from 
	 * of the end of string "repository"
	 * fills the Vector pathVector - a vector representation of nodePath  
	 */
	private String fileToNode(String path) {
		Stack stk = new Stack();
		path = path + "/";
		int end =path.length() -1;
		int nextBegin = 0;
		String subresult = "";
		String result = "";
		for(int pos = nextBegin ; pos < end ; pos = nextBegin + 1) {
			nextBegin = path.indexOf("/",pos);
			subresult = path .substring(pos, nextBegin );
			stk.push(subresult);
			result = result + subresult + "/";
		}
		int last = stk.size() - 1;
		for (int i = 0; i <= last; i++) {
			pathVector.add(stk.pop());
		}
		return result ;
	}// fileToNode method

	void setKind(String kind) {nodeKind = kind;}
	void setPlatform(String platform) {binaryPlatform = platform;}

	public String toString() {return nodeName;}
	String getNodePath() {return nodePath;}
	String getFileName() {return fileName;}
	String getFilePath() {return filePath;}
	String getKind() {return nodeKind;}
	String getPlatform() {return binaryPlatform;}
	String getPath() {
		int end = pathVector.size();
		String nome = ""; 
		for (int i = 0; i < end; i++) {nome = nome + "/" + pathVector.elementAt(i);}
		return nome; 
	}
	
	boolean isApp() {return(nodeKind.equals("_appDir"));}
	boolean isAppDesc() {return(nodeKind.equals("_appDescFile"));}
	boolean isBinary() {return(nodeKind.equals("_binaryFile"));}
	boolean isCommonDir() {return(nodeKind.equals("_commonDir"));}
	boolean isRoot() {return(nodeKind.equals("_root"));}
	boolean isCommonFile() {return(nodeKind == "_commonFile");}

}// TreeNamePathHolder class 


//===================================================
class RepositoryTreeModelListener implements TreeModelListener {
	public void treeNodesChanged(TreeModelEvent e) {
		DefaultMutableTreeNode node;
		node = (DefaultMutableTreeNode) (e.getTreePath().getLastPathComponent());
		/*
		 * If the event lists children, then the changed
		 * node is the child of the node we've already
		 * gotten.  Otherwise, the changed node and the
		 * specified node are the same.
		 */
		try {
			int index = e.getChildIndices()[0];
			node = (DefaultMutableTreeNode)
				   (node.getChildAt(index));
		} catch (NullPointerException exc) {}
	}// treeNodesChanged method

	public void treeNodesInserted(TreeModelEvent e) {
		DefaultMutableTreeNode node;
		node = (DefaultMutableTreeNode) (e.getTreePath().getLastPathComponent());
		/*
		 * If the event lists children, then the changed
		 * node is the child of the node we've already
		 * gotten.  Otherwise, the changed node and the
		 * specified node are the same.
		 */
		try {
			int index = e.getChildIndices()[0];
			node = (DefaultMutableTreeNode)
				   (node.getChildAt(index));
		} catch (NullPointerException exc) {}
	}// treeNodesInserted method

	public void treeNodesRemoved(TreeModelEvent e) {
	}

	public void treeStructureChanged(TreeModelEvent e) {
	}
} // class RepositoryTreeModelListener



