package clusterView;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.image.*;
import java.util.HashMap;
import java.util.LinkedList;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.WindowConstants;

import java.io.*;
import org.jfree.data.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.*;


/**
 *  Implements the ClusterView Historical frames that holds historical 
 *information graphics regarding the machines in an InteGrade cluster in
 * recent period.
 * @author Alexandre Csar Tavares vidal
 * @version 1.0
*/

public class HistoricalFrame extends JInternalFrame {

    private JPanel panel;
    private Controller myManager;

    private XYSeries cpuSeries;
    private XYSeries ramSeries;
    private XYSeries diskSeries;
    private JFreeChart chart;

    private BufferedImage image;
    private JLabel labelChart;


    public HistoricalFrame(String title, Controller ctrl, boolean resizable, boolean closable, boolean maximizable, boolean iconifiable){

        super(title, resizable, closable, maximizable, iconifiable);
	System.out.println("Historical Frame:");
        this.setSize(770, 350);
        this.panel = new JPanel();
        this.getContentPane().add(panel);
        this.setVisible(true);
	this.myManager = ctrl;
	this.labelChart = new JLabel();
	this.panel.add(this.labelChart,null);

	this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

    }

    public void updateHistoricalPanel(HistoricalNode hNode){
	LinkedList properties = hNode.getNodeProperties();
	NodeFrame nFrame = myManager.getFNode(hNode.getNodeName());
	boolean cpuBool = nFrame.cpuIsSelected();
	boolean ramBool = nFrame.ramIsSelected();
	boolean diskBool = nFrame.diskIsSelected();

	if ( !cpuBool && !ramBool && !diskBool )
		    dispose();

	this.cpuSeries = new XYSeries("CPU usage");
	if (cpuBool) {
	    for(int i = 0; i < properties.size(); i++) {
		this.cpuSeries.add(i, ((Usage)properties.get(i)).getCpuUsage()); 
	    } 
	}

	this.ramSeries = new XYSeries("RAM usage");
	if (ramBool) {
	    for(int i = 0; i < properties.size(); i++) 
	        this.ramSeries.add(i, ((Usage)properties.get(i)).getRamUsage());
	}

	this.diskSeries = new XYSeries("DISK usage");
	if (diskBool) {	
	    for(int i = 0; i < properties.size(); i++) 
		this.diskSeries.add(i, ((Usage)properties.get(i)).getDiskUsage());
	}

	XYSeriesCollection xyDataset = new XYSeriesCollection();
	xyDataset.addSeries(cpuSeries);
	xyDataset.addSeries(ramSeries);
	xyDataset.addSeries(diskSeries);

	this.chart = ChartFactory.createXYLineChart
	    ("Usage",
	     "Last refreshes",
	     "Percentual",
	     xyDataset,
	     PlotOrientation.VERTICAL,
	     true,false,false);
        this.image = chart.createBufferedImage(740,300);
	this.labelChart.setIcon(new ImageIcon(image));
    }
}






