package clusterView;

import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.WindowConstants;

/**
 * Implements a TabbededPane and its internal windows with 
 * information regarding the machines in an InteGrade cluster.
 * @author Alexandre Csar Tavares Vidal
 * @version 1.0
 */

public class NodeFrame extends JInternalFrame 
    implements ItemListener {

    private InternalPanel panel;

    private JMenuBar menuBar;
    private JCheckBoxMenuItem cpuCbMenuItem, diskCbMenuItem, ramCbMenuItem;
    private Controller myManager;


    // utilities

    public boolean cpuIsSelected() {
	return cpuCbMenuItem.isSelected();
    }

    public boolean ramIsSelected() {
	return ramCbMenuItem.isSelected();
    }

    public boolean diskIsSelected() {
	return diskCbMenuItem.isSelected();
    }


    // constructor
    public NodeFrame(String title, Controller ctrl, boolean resizable, boolean closable, boolean maximizable, boolean iconifiable){
        super(title, resizable, closable, maximizable, iconifiable);
        this.setPreferredSize(new Dimension(800,120));
	this.panel = new InternalPanel(ctrl);
	this.getContentPane().add(panel);
	this.setJMenuBar(createMenuBar());
	this.myManager = ctrl;
	
        this.setVisible(true);
	this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    }


    public void updateInternalPanel(HashMap specs){
        this.panel.updateBarPanel(specs);
        this.panel.updateStaticInfoPanel(specs);
        this.panel.updateDynamicInfoPanel(specs);
    }


    protected JMenuBar createMenuBar() {
        this.menuBar = new JMenuBar();

	this.cpuCbMenuItem = new JCheckBoxMenuItem("CPU");
	this.cpuCbMenuItem.addItemListener(this);
	this.menuBar.add(cpuCbMenuItem);

	this.ramCbMenuItem = new JCheckBoxMenuItem("RAM");
	this.ramCbMenuItem.addItemListener(this);
	this.menuBar.add(ramCbMenuItem);

	this.diskCbMenuItem = new JCheckBoxMenuItem("Disk");
	this.diskCbMenuItem.addItemListener(this);
	this.menuBar.add(diskCbMenuItem);

	this.cpuCbMenuItem.setEnabled(true);
	this.ramCbMenuItem.setEnabled(true);
	this.diskCbMenuItem.setEnabled(true);
  
        return menuBar;
    }


    // The actions to be performed when a JCheckBox is selected

    public void itemStateChanged(ItemEvent e) {
	    this.myManager.updateOneHistoricalFrame(getTitle());	    
    }

}


class InternalPanel extends JPanel{

    private BorderLayout internalPanelLayout;
    private JPanel barPanel, dynamicInfo, staticInfo;
    private JProgressBar cpuUsageBar, memoryUsageBar, swapUsageBar;
    private JProgressBar filesUsageBar,  diskUsageBar;
    private TitledBorder barPanelBorder, staticInfoBorder, dynamicInfoBorder;
    private JLabel cpuLabel, memoryLabel, swapLabel, diskLabel , filesLabel;
    private JLabel osVersionLabel, hostNameLabel, osNameLabel;
    private JLabel processorClockLabel, processorNameLabel;
    private JLabel totalRAMLabel, totalSwapLabel, totalFSLabel;
    private JLabel totalFilesLabel, freeFSLabel, freeSwapLabel, freeRAMLabel;
    private JLabel freeFilesLabel, cpuUsageLabel;


    private Controller myManager;
    
    public InternalPanel (Controller ctrl) {
        
	this.internalPanelLayout = new BorderLayout();  
        this.barPanel = new JPanel();
        this.dynamicInfo = new JPanel();
        this.staticInfo = new JPanel();
        this.cpuUsageBar = new JProgressBar();
        this.memoryUsageBar = new JProgressBar();
        this.swapUsageBar = new JProgressBar();
        this.diskUsageBar = new JProgressBar();
        this.filesUsageBar = new JProgressBar();

	this.myManager = ctrl;

        try {
            internalPanelInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    


    private void internalPanelInit() throws Exception {
        
        /* --------------------------- BarPanel ----------------------------*/
        
        this.barPanelBorder = new TitledBorder(BorderFactory.createEtchedBorder(
            Color.white,new Color(156, 156, 158)),"Use Percentage");
        this.barPanel.setBorder(barPanelBorder);
        this.barPanel.setPreferredSize(new Dimension(220, 30));
        this.barPanel.setLayout(null);
        // CPU Label
        this.cpuLabel = new JLabel("CPU:");
        this.cpuLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.cpuLabel.setBounds(new Rectangle(11, 30, 46, 15));
        // RAM Memory Label
        this.memoryLabel = new JLabel("Memory:");
        this.memoryLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.memoryLabel.setBounds(new Rectangle(11, 51, 55, 15));
        // Swap Label
        this.swapLabel = new JLabel("Swap:");
        this.swapLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.swapLabel.setBounds(new Rectangle(11, 72, 55, 15));
        // Disk Label
        this.diskLabel = new JLabel("Disk:");
        this.diskLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.diskLabel.setBounds(new Rectangle(11, 93, 55, 15));
        // Files Label
        this.filesLabel = new JLabel("Files:");
        this.filesLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.filesLabel.setBounds(new Rectangle(11, 114, 55, 15));
        // CPU Usage Bar
        this.cpuUsageBar.setOrientation(JProgressBar.HORIZONTAL);
        this.cpuUsageBar.setPreferredSize(new Dimension(110, 16));
        this.cpuUsageBar.setMaximum(100);
        this.cpuUsageBar.setStringPainted(true);
        this.cpuUsageBar.setBounds(new Rectangle(80, 29, 110, 16));
        // RAM Memory Usage Bar
        this.memoryUsageBar.setOrientation(JProgressBar.HORIZONTAL);
        this.memoryUsageBar.setPreferredSize(new Dimension(110, 16));
        this.memoryUsageBar.setMaximum(100);        
        this.memoryUsageBar.setStringPainted(true);
        this.memoryUsageBar.setBounds(new Rectangle(80, 50, 109, 16));
        // Swap Usage Bar
        this.swapUsageBar.setOrientation(JProgressBar.HORIZONTAL);
        this.swapUsageBar.setPreferredSize(new Dimension(110, 16));
        this.swapUsageBar.setMaximum(100);
        this.swapUsageBar.setStringPainted(true);
        this.swapUsageBar.setBounds(new Rectangle(80, 71, 109, 16));
        // Disk Usage Bar
        this.diskUsageBar.setOrientation(JProgressBar.HORIZONTAL);
        this.diskUsageBar.setPreferredSize(new Dimension(110, 16));
        this.diskUsageBar.setMaximum(100);
        this.diskUsageBar.setStringPainted(true);
        this.diskUsageBar.setBounds(new Rectangle(80, 92, 109, 16));
        // Files Usage Bar
        this.filesUsageBar.setOrientation(JProgressBar.HORIZONTAL);
        this.filesUsageBar.setPreferredSize(new Dimension(110, 16));
        this.filesUsageBar.setMaximum(100);
        this.filesUsageBar.setStringPainted(true);
        this.filesUsageBar.setBounds(new Rectangle(80, 113, 109, 16));

        this.barPanel.add(cpuLabel, null);
        this.barPanel.add(cpuUsageBar, null);
        this.barPanel.add(memoryLabel, null);
        this.barPanel.add(memoryUsageBar, null);
        this.barPanel.add(swapLabel, null);
        this.barPanel.add(swapUsageBar, null);
        this.barPanel.add(diskLabel, null);
        this.barPanel.add(diskUsageBar, null);
        this.barPanel.add(filesLabel, null);
        this.barPanel.add(filesUsageBar, null);

        this.barPanelBorder.setTitleFont(new java.awt.Font("SansSerif", 1, 11));
        
        /* --------------------------- StaticInfoPanel ----------------------------*/
        this.staticInfoBorder = new TitledBorder(BorderFactory.createEtchedBorder(
            Color.white,new Color(156, 156, 158)),"Static Info");
        this.staticInfo.setBorder(staticInfoBorder);
        this.staticInfo.setLayout(null);
        this.staticInfo.setPreferredSize(new Dimension(200, 30));
		
	// Host Name Label
        this.hostNameLabel = new JLabel("Host Name:");
        this.hostNameLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.hostNameLabel.setBounds(new Rectangle(11, 30, 190, 15));
        // OS Name Label
        this.osNameLabel = new JLabel("OS Name:");
        this.osNameLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.osNameLabel.setBounds(new Rectangle(11, 51, 190, 15));
        // OS Version Label
        this.osVersionLabel = new JLabel("OS Version:");
        this.osVersionLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.osVersionLabel.setBounds(new Rectangle(11, 72, 190, 15));
        // Processor Name Label
        this.processorNameLabel = new JLabel("Processor Name:");
        this.processorNameLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.processorNameLabel.setBounds(new Rectangle(11, 93, 190, 15));
	this.staticInfo.add(processorNameLabel, null);

	

	// Processor Clock Label
        this.processorClockLabel = new JLabel("Processor Clock:");
        this.processorClockLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.processorClockLabel.setBounds(new Rectangle(220, 30, 190, 15));
        // Total RAM Label
        this.totalRAMLabel = new JLabel("Total RAM:");
        this.totalRAMLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.totalRAMLabel.setBounds(new Rectangle(220, 51, 190, 15));
        // Total Swap Label
        this.totalSwapLabel = new JLabel("Total Swap:");
        this.totalSwapLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.totalSwapLabel.setBounds(new Rectangle(220, 72, 190, 15));
        // Total Disk Label
        this.totalFSLabel = new JLabel("Total Disk:");
        this.totalFSLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.totalFSLabel.setBounds(new Rectangle(220, 93, 190, 15));
        // Total Files Label
        this.totalFilesLabel = new JLabel("Total Files:");
        this.totalFilesLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.totalFilesLabel.setBounds(new Rectangle(220, 114, 190, 15));

	this.staticInfo.add(hostNameLabel, null);
        this.staticInfo.add(osNameLabel, null);
        this.staticInfo.add(osVersionLabel, null); 
	this.staticInfo.add(processorNameLabel, null);
        this.staticInfo.add(processorClockLabel, null);
        this.staticInfo.add(totalRAMLabel, null);
        this.staticInfo.add(totalSwapLabel, null);
        this.staticInfo.add(totalFSLabel, null);
        this.staticInfo.add(totalFilesLabel, null);

        this.staticInfoBorder.setTitleFont(new java.awt.Font("SansSerif", 1, 11));

        /* --------------------------- DynamicInfoPanel ----------------------------*/
        this.dynamicInfoBorder = new TitledBorder(BorderFactory.createEtchedBorder(
            Color.white,new Color(156, 156, 158)),"Dynamic Info");
        this.dynamicInfo.setBorder(dynamicInfoBorder);
        this.dynamicInfo.setPreferredSize(new Dimension(150, 40));
        this.dynamicInfo.setLayout(null);
        // Free Ram Label
        this.freeRAMLabel = new JLabel("Free RAM:");
        this.freeRAMLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.freeRAMLabel.setBounds(new Rectangle(10, 25, 190, 15));
        // Free Swap Label
        this.freeSwapLabel = new JLabel("Freel Swap:");
        this.freeSwapLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.freeSwapLabel.setBounds(new Rectangle(10, 47, 190, 15));
        // Free FS Label
        this.freeFSLabel = new JLabel("Free FS:");
        this.freeFSLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.freeFSLabel.setBounds(new Rectangle(10, 69, 190, 15));
        // Free Files Label
        this.freeFilesLabel = new JLabel("Free Files:");
        this.freeFilesLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        this.freeFilesLabel.setBounds(new Rectangle(10, 90, 190, 15));
        // CPU Usage Label
        this.cpuUsageLabel = new JLabel("CPU Usage:");
        this.cpuUsageLabel.setBounds(new Rectangle(10, 112, 190, 15));
        this.cpuUsageLabel.setFont(new java.awt.Font("SansSerif", 0, 11));

        this.dynamicInfo.add(freeRAMLabel, null);
        this.dynamicInfo.add(freeSwapLabel, null);
        this.dynamicInfo.add(freeFSLabel, null);
        this.dynamicInfo.add(freeFilesLabel, null);
        this.dynamicInfo.add(cpuUsageLabel, null);
        
        this.dynamicInfoBorder.setTitleFont(new java.awt.Font("SansSerif", 1, 11));
        
        /* --------------------------- MainPanel ----------------------------*/
        this.setForeground(Color.black);
        this.setLayout(internalPanelLayout);
        this.add(barPanel, BorderLayout.WEST);
	this.add(staticInfo, BorderLayout.CENTER);
	this.add(dynamicInfo,  BorderLayout.EAST);
        this.setPreferredSize(new Dimension(800, 120));
    }
    
    public int roundPercentage(HashMap specs, String partField, String totalField){
        float total = ((Integer) specs.get(totalField)).floatValue();
        float part = ((Integer) specs.get(partField)).floatValue();
        //We want the usage percentage. So, subtract
        return (int) Math.round(100 *(1 - (part / total)) );
    }

    public void updateBarPanel(HashMap specs){

        cpuUsageBar.setValue( (int) Math.round(Float.parseFloat((String)specs.get("cpuUsage"))));
        memoryUsageBar.setValue(myManager.roundPercentage(specs, "freeRAM", "totalRAM"));
        swapUsageBar.setValue(myManager.roundPercentage(specs, "freeSwap", "totalSwap"));
        diskUsageBar.setValue(myManager.roundPercentage(specs, "fsFree", "fsTotal"));
        filesUsageBar.setValue(myManager.roundPercentage(specs, "filesFree", "filesTotal"));
    
    }
    
    public void updateStaticInfoPanel(HashMap specs){

	this.hostNameLabel.setText("Host Name: " + specs.get("hostName"));
	this.osNameLabel.setText("OS Name: " + specs.get("osName"));
        this.osVersionLabel.setText("OS Version: " + specs.get("osVersion"));
        this.processorNameLabel.setText("Processor Name: " + specs.get("processorName"));
        this.processorClockLabel.setText("Processor Clock: " + specs.get("processorMhz"));
        this.totalRAMLabel.setText("Total RAM: " + specs.get("totalRAM"));
        this.totalSwapLabel.setText("Total Swap: " + specs.get("totalSwap"));
        this.totalFSLabel.setText("Total Disk: " + specs.get("fsTotal"));
        this.totalFilesLabel.setText("Total Files: " + specs.get("filesTotal"));
    }

    public void updateDynamicInfoPanel(HashMap specs){
        
        this.freeRAMLabel.setText("Free RAM: " + specs.get("freeRAM"));
        this.freeSwapLabel.setText("Free Swap: " + specs.get("freeSwap"));
        this.freeFSLabel.setText("Free FS: " + specs.get("fsFree"));
        this.freeFilesLabel.setText("Free Files: " + specs.get("filesFree"));
        this.cpuUsageLabel.setText("CPU Usage: " + specs.get("cpuUsage"));
       
    }

}



